/**
 * <copyright>
 *
 * Copyright (c) 2002 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * plugins/org.eclipse.emf.ecore.xmi/src/org/eclipse/emf/ecore/xmi/impl/SAXWrapper.java, emf.ecore.xmi, org.eclipse.111, 20031120_1149WL
 * @version 1.1 11/20/03
 */
package org.eclipse.emf.ecore.xmi.impl;

import org.eclipse.emf.ecore.xmi.XMIException;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class wraps an XMLHandler with a SAX DefaultHandler.
 */
public class SAXWrapper extends DefaultHandler
{
  protected XMLHandler handler;
  
  /**
   * Constructor for SAXWrapper.
   */
  public SAXWrapper(XMLHandler handler)
  {
    super();
    this.handler = handler;
  }
    public void setDocumentLocator(Locator locator)
    {
      handler.setLocator(locator);
    }

    public void startDocument() throws SAXException
    {
      handler.startDocument();
    }

    public void endDocument() throws SAXException
    {
      handler.endDocument();
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
      handler.startPrefixMapping(prefix, uri);
    }

    public void endPrefixMapping(String prefix) throws SAXException
    {
      handler.endPrefixMapping(prefix);
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
    {
      handler.setAttributes(attributes);
      handler.startElement(uri, localName, qName);
    }

    public void endElement (String uri, String localName, String qName) throws SAXException
    {
      handler.endElement(uri, localName, qName);
    }

    public void warning (SAXParseException e) throws SAXException
    {
      XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
      handler.warning(xmi);
    }

    public void error (SAXParseException e) throws SAXException
    {
      XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
      handler.error(xmi);
    }

    public void fatalError (SAXParseException e) throws SAXException
    {
      XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
      handler.fatalError(xmi);
      throw e;
    }

    public void characters (char ch[], int start, int length) throws SAXException
    {
      handler.characters(ch, start, length);
    }

    public void ignorableWhitespace (char ch[], int start, int length) throws SAXException
    {
      // handler.ignorableWhitespace(ch, start, length);
    }

    public void processingInstruction (String target, String data) throws SAXException
    {
      // handler.processingInstruction(target, data);
    }

    public void skippedEntity (String name) throws SAXException
    {
      // handler.skippedEntity(name);
    }

    public InputSource resolveEntity(String publicId, String systemId) throws SAXException
    {
      // handler.resolveEntity(publicId, systemId);
      return null;
    }

    public void notationDecl(String name, String publicId, String systemId) throws SAXException
    {
      // handler.notationDecl(name, publicId, systemId);
    }

    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException
    {
      // handler.unparsedEntityDecl(name, publicId, systemId, notationName);
    }
}
